<?php

/**
 * @file
 * Base class for the plugins.
 */

namespace Drupal\dream_fields;

use Drupal\Component\Plugin\PluginBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityManagerInterface;
use Drupal\Core\Field\FieldTypePluginManagerInterface;

/**
 * Base class for the plugins.
 */
abstract class DreamFieldPluginBase extends PluginBase implements DreamFieldPluginInterface {

  /**
   * Array with storage settings.
   *
   * @var array
   */
  protected $field_storage_values = [];

  /**
   * Array with field settings.
   *
   * @var array
   */
  protected $field_values = [];

  /**
   * Array with field display settings.
   *
   * @var array
   */
  protected $field_display_values = [];

  /**
   * The entity type.
   *
   * @var string
   */
  protected $entity_type;

  /**
   * The name of the entity type.
   *
   * @var string
   */
  protected $entityTypeId;

  /**
   * The entity bundle.
   *
   * @var string
   */
  protected $bundle;

  /**
   * The entity manager.
   *
   * @var \Drupal\Core\Entity\EntityManager
   */
  protected $entityManager;

  /**
   * The field type plugin manager.
   *
   * @var \Drupal\Core\Field\FieldTypePluginManagerInterface
   */
  protected $fieldTypePluginManager;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new FieldStorageAddForm object.
   *
   * @param \Drupal\Core\Entity\EntityManagerInterface $entity_manager
   *   The entity manager.
   * @param \Drupal\Core\Field\FieldTypePluginManagerInterface $field_type_plugin_manager
   *   The field type plugin manager.
   * @param \Drupal\Core\Entity\Query\QueryFactory $query_factory
   *   The entity query factory.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition, EntityManagerInterface $entity_manager, FieldTypePluginManagerInterface $field_type_plugin_manager, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->entityManager = $entity_manager;
    $this->fieldTypePluginManager = $field_type_plugin_manager;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity.manager'),
      $container->get('plugin.manager.field.field_type'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    return $this->pluginDefinition['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function getDefinitions() {
    return [];
  }

  /**
   * Create a machine name.
   */
  protected function createMachineName($label) {
    $machine_name = strtolower($label);
    $machine_name = preg_replace('/[^a-z0-9_]+/', '_', $machine_name);
    return preg_replace('/_+/', '_', $machine_name);
  }

  /**
   * Create a prefixed field name.
   */
  protected function createFieldMachineName($label) {
    $machine_name = $this->createMachineName($label);
    $field_name = $this->configFactory->get('field_ui.settings')->get('field_prefix') . $machine_name;
    return substr($field_name, 0, 31);
  }

  /**
   * Create the field storage and field.
   */
  protected function createFieldStorageAndField() {
    // Merge some defaults.
    $this->field_storage_values += [
      'field_name' => $this->field_name,
      'entity_type' => $this->entityTypeId,
      'translatable' => TRUE,
      'cardinality' => 1,
      'settings' => [],
    ];

    $this->field_values += [
      'field_name' => $this->field_name,
      'entity_type' => $this->entityTypeId,
      'bundle' => $this->bundle,
      'translatable' => FALSE,
      'settings' => [],
    ];

    $this->field_display_values += [
      'label' => 'above',
    ];

    $this->entityManager->getStorage('field_storage_config')->create($this->field_storage_values)->save();
    $field = $this->entityManager->getStorage('field_config')->create($this->field_values);
    $field->save();

    entity_get_form_display($this->entityTypeId, $this->bundle, 'default')
      ->setComponent($this->field_name, ['type' => $this->widget_id])
      ->save();

    entity_get_display($this->entityTypeId, $this->bundle, 'default')
        ->setComponent($this->field_name, $this->field_display_values)
        ->save();
  }

}
