<?php

/**
 * @file
 * Dream fields module.
 */

namespace Drupal\dream_fields\Form;

use Drupal\Core\Form\FormStateInterface;
use Drupal\field_ui\FieldUI;

/**
 * Class AddAField.
 *
 * @package Drupal\dream_fields\Form
 */
class AddAField extends \Drupal\field_ui\Form\FieldStorageAddForm {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'add_a_field';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $entity_type_id = NULL, $bundle = NULL) {
    if (!$form_state->get('entity_type_id')) {
      $form_state->set('entity_type_id', $entity_type_id);
    }
    if (!$form_state->get('bundle')) {
      $form_state->set('bundle', $bundle);
    }

    $this->entityTypeId = $form_state->get('entity_type_id');
    $this->bundle = $form_state->get('bundle');
    $bundles = $this->entityManager->getBundleInfo($this->entityTypeId);

    $form_title = $this->t('Quickly add a field to %bundle', array(
      '%bundle' => $bundles[$this->bundle]['label'],
    ));
    $form['#title'] = $form_title;

    // @todo Add a link.
    $form['intro'] = [
      '#type' => 'markup',
      '#markup' => $this->t('If you want to re-use an existing field, use the regular "Add field" action.'),
      '#weight' => -50,
    ];

    $form['label'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Label'),
      '#required' => TRUE,
      '#weight' => -20,
    ];

    $form['required'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Check this if the field is mandatory'),
      '#weight' => -5,
    ];

    $plugin_manager = \Drupal::service('plugin.manager.dream_fields');
    $new_field_options = [];
    $dream_fields_definitions = $plugin_manager->getDefinitions();
    foreach ($dream_fields_definitions as $plugin_id => $plugin) {
      // Add it to the options.
      $description = isset($plugin['description']) ? $plugin['description'] : '';
      // @todo Ugly code, needs twig love.
      $new_field_options[$plugin_id] = [
        'title' => $plugin['label'],
        'description' => $description,
        'preview' => isset($plugin['preview']) ? '/' . drupal_get_path('module', $plugin['provider']) . '/' . $plugin['preview'] : NULL,
      ];

      // Build the form.
      $form_data = $plugin_manager->createInstance($plugin_id)->getForm();
      if ($form_data) {
        $form['new_field_info'][$plugin_id] = [
          '#type' => 'container',
          '#states' => [
            'visible' => [
              ':input[name="new_field"]' => ['value' => $plugin_id],
            ],
          ],
        ];
        foreach ($form_data as $form_key => $form_info) {
          $form['new_field_info'][$plugin_id][$form_key] = $form_info;
        }
      }
    }

    asort($new_field_options);

    $form['new_field'] = [
      '#type' => 'radios_with_image',
      '#title' => $this->t('Select the field you want to create'),
      '#description' => $this->t('Depending on your selection, you might have to specify more options.'),
      '#options' => $new_field_options,
      '#required' => TRUE,
      '#weight' => -2,
    ];

    $form['actions'] = [
      '#type' => 'actions',
      '#states' => [
        'visible' => [
          ':input[name="new_field"]' => ['!value' => FALSE],
        ],
      ],
    ];
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Create field'),
      '#button_type' => 'primary',
    ];

    $form['#attached']['library'][] = 'dream_fields/dream-fields-form';

    $form['#tree'] = TRUE;
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    if (!$form_state->getValue('new_field')) {
      $form_state->setErrorByName('new_field', $this->t('You need to select a field.'));
    }

    if (!$form_state->getValue('label')) {
      $form_state->setErrorByName('label', $this->t('You need to provide a label.'));
    }

    // Let the plugin validate.
    $values = $form_state->getValues();
    $plugin_id = $values['new_field'];
    $plugin_manager = \Drupal::service('plugin.manager.dream_fields');
    $plugin = $plugin_manager->createInstance($plugin_id);

    // Only send data the plugin needs.
    $data = [];
    if (isset($values['new_field_info'][$plugin_id])) {
      $data = $values['new_field_info'][$plugin_id];
    }
    $plugin->validateForm($data, $this->entityTypeId, $this->bundle);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $values = $form_state->getValues();
    $plugin_id = $values['new_field'];
    $plugin_manager = \Drupal::service('plugin.manager.dream_fields');
    $plugin = $plugin_manager->createInstance($plugin_id);

    // Only send data the plugin needs.
    $data = [];
    if (isset($values['new_field_info'][$plugin_id])) {
      $data = $values['new_field_info'][$plugin_id];
    }
    $plugin->saveForm($values['label'], $values['required'], $data, $this->entityTypeId, $this->bundle);

    // Redirect.
    $entity_type = $this->entityManager->getDefinition($this->entityTypeId);
    $route_parameters = [
      'route_name' => "entity.{$this->entityTypeId}.field_ui_fields",
    ] + FieldUI::getRouteBundleParameter($entity_type, $this->bundle);
    $form_state->setRedirect("entity.{$this->entityTypeId}.field_ui_fields", $route_parameters);

    // @todo Check wording and add direct links.
    drupal_set_message('Your field is created and ready to be used, can find more advanced options on this page or use the tabs at the top.');
  }

}
