<?php

/**
 * @file
 * Base class for all 3 list plugins.
 */

namespace Drupal\dream_fields\Plugin\DreamField;

use Drupal\Core\Language\LanguageInterface;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Component\Utility\Html;
use Drupal\Core\StringTranslation\StringTranslationTrait;

/**
 * Base class for all 3 list plugins.
 */
class DreamFieldList extends \Drupal\dream_fields\DreamFieldPluginBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   *
   * @see buildForm
   */
  public function getForm() {
  }

  /**
   * Build the form.
   */
  public function buildForm($key) {
    $form = [];

    $vocabularies = [];
    foreach (Vocabulary::loadMultiple() as $vocabulary) {
      $vocabularies[$vocabulary->id()] = Html::escape($this->entityManager->getTranslationFromContext($vocabulary)->label());
    }

    $form['list_type'] = [
      '#type' => 'radios',
      '#title' => $this->t('Select which data you want to use'),
      '#options' => [
        'manual' => 'Enter data manually',
        'link_to_voc' => 'Select an existing vocabulary',
        'create_voc' => 'Create a new vocabulary',
        'link_to_entity' => 'Select the type of content to link to',
      ],
      '#attributes' => [
        'class' => ['dream-fields--' . $key],
      ],
    ];

    $form['list_type_manual'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Enter here your data'),
      '#description' => $this->t('Enter each option on a separate line.'),
      '#states' => [
        'visible' => [
          'input.dream-fields--' . $key => ['value' => 'manual'],
        ],
        'required' => [
          'input.dream-fields--' . $key => ['value' => 'manual'],
        ],
      ],
    ];

    $form['list_type_create_voc'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Enter the name of the vocabulary you want to be created'),
      '#states' => [
        'visible' => [
          'input.dream-fields--' . $key => ['value' => 'create_voc'],
        ],
        'required' => [
          'input.dream-fields--' . $key => ['value' => 'create_voc'],
        ],
      ],
    ];

    $form['list_type_link_to_voc'] = [
      '#type' => 'select',
      '#options' => $vocabularies,
      '#title' => $this->t('Select the vocabulary you want to use'),
      '#states' => [
        'visible' => [
          'input.dream-fields--' . $key => ['value' => 'link_to_voc'],
        ],
        'required' => [
          'input.dream-fields--' . $key => ['value' => 'link_to_voc'],
        ],
      ],
    ];

    $bundle_info = \Drupal::entityManager()->getBundleInfo('node');
    $bundle_options = [];
    foreach ($bundle_info as $bundle_key => $bundle_label) {
      $bundle_options[$bundle_key] = $bundle_label['label'];
    }

    $form['list_type_link_to_entity'] = [
      '#type' => 'select',
      '#options' => $bundle_options,
      '#title' => $this->t('Select the content type to link to.'),
      '#states' => [
        'visible' => [
          'input.dream-fields--' . $key => ['value' => 'link_to_entity'],
        ],
        'required' => [
          'input.dream-fields--' . $key => ['value' => 'link_to_entity'],
        ],
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm() {
  }

  /**
   * {@inheritdoc}
   */
  public function saveForm($label, $required, $values, $entityTypeId, $bundle) {
    $this->field_name = $this->createFieldMachineName($label);
    $this->entityTypeId = $entityTypeId;
    $this->bundle = $bundle;

    $this->field_storage_values = [
      'field_name' => $this->field_name,
      'entity_type' => $entityTypeId,
      'type' => 'list_string',
      'translatable' => TRUE,
      'cardinality' => $this->cardinality,
      'settings' => [],
    ];
    $this->field_values = [
      'field_name' => $this->field_name,
      'entity_type' => $entityTypeId,
      'bundle' => $bundle,
      'label' => $label,
      'translatable' => FALSE,
      'required' => $required,
      'settings' => [],
    ];
    $this->field_display_values += [
      'label' => 'inline',
    ];

    switch ($values['list_type']) {
      case 'manual':
        $list = $values['list_type_manual'];
        $list = explode("\n", $list);
        $list = array_map('trim', $list);
        $list = array_filter($list, 'strlen');
        foreach ($list as $value) {
          $value = explode("|", $value);
          $allowed_values[$value[0]] = isset($value[1]) ? $value[1] : $value[0];
        }
        $this->field_storage_values['settings']['allowed_values'] = $allowed_values;
        break;

      case 'create_voc':
        $vocabulary = Vocabulary::create([
          'name' => $values['list_type_create_voc'],
          'vid' => 'voc_' . $this->createMachineName($label),
          'langcode' => LanguageInterface::LANGCODE_NOT_SPECIFIED,
          'weight' => 0,
        ]);
        $vocabulary->save();

        $this->field_storage_values['type'] = 'entity_reference';
        $this->field_storage_values['settings']['target_type'] = 'taxonomy_term';

        $this->field_values['link_to_entity'] = TRUE;
        $this->field_values['settings'] = [
          'handler' => 'default',
          'handler_settings' => [
            'target_bundles' => [
              $vocabulary->id() => $vocabulary->id(),
            ],
          ],
          'sort' => [
            'field' => 'title',
            'direction' => 'ASC',
          ],
        ];
        break;

      case 'link_to_voc':
        $vid = $values['list_type_link_to_voc'];
        $this->field_storage_values['type'] = 'entity_reference';
        $this->field_storage_values['settings']['target_type'] = 'taxonomy_term';

        $this->field_values['link_to_entity'] = TRUE;
        $this->field_values['settings'] = [
          'handler' => 'default',
          'handler_settings' => [
            'target_bundles' => [
              $vid => $vid,
            ],
          ],
        ];
        break;

      case 'link_to_entity':
        $this->field_storage_values['type'] = 'entity_reference';
        $this->field_storage_values['settings']['target_type'] = 'node';
        $this->field_values['link_to_entity'] = TRUE;

        $this->field_values['link_to_entity'] = TRUE;
        $this->field_values['settings'] = [
          'handler' => 'default:node',
          'handler_settings' => [
            'target_bundles' => [
              $values['list_type_link_to_entity'] => $values['list_type_link_to_entity'],
            ],
          ],
        ];
        break;
    }

    $this->createFieldStorageAndField();
  }

}
